#!/usr/bin/env python
# Sedit tiny editor by Damien Tanner (dctanner@xoasis.com)      
#
#    Sedit is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Sedit is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Sedit; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import *
import string

class Sedit:
	def __init__(self):
		self.VERSION = '0.0.4'
		self.window = GtkWindow(WINDOW_TOPLEVEL)
		self.window.connect("delete_event", self.quit)
		self.window.set_title("Sedit " + self.VERSION)
		if screen_width() > 300:
			self.window.set_usize(280, 240)
		else:
			self.window.set_usize(240, 280)
		self.window.show()

		self.vbox = GtkVBox()
		self.window.add(self.vbox)
		self.vbox.show()

		hbox = GtkHBox()
		self.vbox.pack_start(hbox, expand=FALSE)
		hbox.show()

		self.toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar.set_button_relief(RELIEF_NONE)
		self.toolbar.set_space_style(TOOLBAR_SPACE_LINE)
		self.toolbar.set_space_size(10)
		hbox.pack_start(self.toolbar, expand=TRUE)
		self.toolbar.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/new.xpm")
		self.toolbar.append_item(None, 'Create a new document', None, GtkPixmap(pix, mask), self.new_file)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/open.xpm")
		self.toolbar.append_item(None, 'Open a file', None, GtkPixmap(pix, mask), self.select_open_file)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/save.xpm")
		self.toolbar.append_item(None, 'Save the current file', None, GtkPixmap(pix, mask), self.save_file)

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/close.xpm")
		self.toolbar.append_item(None, 'Close the current file', None, GtkPixmap(pix, mask), self.ask_close_file)

		self.toolbar.append_space()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/cut.xpm")
		self.toolbar.append_item(None, 'Cut the selection', None, GtkPixmap(pix, mask), self.cut_selection)


		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/copy.xpm")
		self.toolbar.append_item(None, 'Copy the selection', None, GtkPixmap(pix, mask), self.copy_selection)


		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/paste.xpm")
		self.toolbar.append_item(None, 'Paste the clipboard', None, GtkPixmap(pix, mask), self.paste_clipboard)


		self.toolbar2 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar2.set_button_relief(RELIEF_NONE)
		self.toolbar2.set_space_size(10)
		hbox.pack_end(self.toolbar2, expand=FALSE)
		self.toolbar2.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, "/usr/share/sedit/pixmaps/exit.xpm")
		self.toolbar2.append_item(None, 'Exit', None, GtkPixmap(pix, mask), self.quit)

		self.notebook = GtkNotebook()
		self.vbox.pack_start(self.notebook)
		self.notebook.show()

		# []{filename, edited, gtk text, gtk label}
		self.file = []

		self.new_file()

	def new_file(self, *args):
		'''Create a blank text tab.'''

		text = GtkText()
		text.connect("changed", self.text_changed)
		text.set_editable(TRUE)
		text.show()

		scroll = GtkScrolledWindow()
		scroll.add(text)
		scroll.show()

		label = GtkLabel('Untitled ' + str(len(self.file)))
		label.show()

		file = {}
		file["filename"] = ''
		file["edited"] = 0
		file["text_widget"] = text
		file["label_widget"] = label

		self.file.append(file)
		self.notebook.append_page(scroll, label)
		self.notebook.set_page(len(self.file) - 1)

	def select_open_file(self, *args):
		'''Open the selected file.'''

		fileselection = GtkFileSelection("file selection dialog")
		fileselection.connect("delete_event", fileselection.destroy)
		fileselection.ok_button.connect("clicked", self.open_file, fileselection)
		fileselection.cancel_button.connect("clicked", fileselection.destroy)
		fileselection.show()

	def open_file(self, args, fileselection):
		'''Select a file to open.'''

		filename =  fileselection.get_filename()
		fileselection.destroy()

		file = open(filename, 'r')
		file_text = file.read()
		file.close()

		text = GtkText()
		text.insert(None,None,None,file_text)
		text.connect("changed", self.text_changed)
		text.set_editable(TRUE)
		text.show()

		scroll = GtkScrolledWindow()
		scroll.add(text)
		scroll.show()

		filename_parts = string.split(filename, '/')
		label = GtkLabel(filename_parts[len(filename_parts) - 1])
		label.show()

		file = {}
		file["filename"] = filename
		file["edited"] = 0
		file["text_widget"] = text
		file["label_widget"] = label

		self.file.append(file)
		self.notebook.append_page(scroll, label)
		self.notebook.set_page(len(self.file) - 1)

	def save_file(self, *args):
		'''Save the current tab.'''

		current_page = self.notebook.get_current_page()
		if self.file[current_page]["edited"] == 1:
			if self.file[current_page]["filename"] == '':
				print "Save as"
				fileselection = GtkFileSelection("file selection dialog")
				fileselection.connect("delete_event", fileselection.destroy)
				fileselection.ok_button.connect("clicked", self.save_file_as, fileselection)
				fileselection.cancel_button.connect("clicked", fileselection.destroy)
				fileselection.show()
			else:
				text = self.file[current_page]["text_widget"].get_chars(0, -1)
				file = open(self.file[current_page]["filename"], 'w')
				file.write(text)
				file.close()

				self.file[current_page]["edited"] = 0
				filename_parts = string.split(self.file[current_page]["filename"], '/')
				self.file[current_page]["label_widget"].set_text(filename_parts[len(filename_parts) - 1])

	def save_file_as(self, args, fileselection):
		'''Save the current tab as a new file.'''

		current_page = self.notebook.get_current_page()
		filename =  fileselection.get_filename()
		fileselection.destroy()

		self.file[current_page]["filename"] = filename

		file = open(filename, 'w')
		file.write(self.file[current_page]["text_widget"].get_chars(0, -1))
		file.close()

		self.file[current_page]["edited"] = 0
		filename_parts = string.split(self.file[current_page]["filename"], '/')
		self.file[current_page]["label_widget"].set_text(filename_parts[len(filename_parts) - 1])

	def ask_close_file(self, *args):
		'''Ask if the user really wants to close the current tab.'''

		current_page = self.notebook.get_current_page()
		if self.file[current_page]["edited"] == 1:
			window = GtkWindow(WINDOW_TOPLEVEL)
			self.ask_close_window = window
			window.connect("delete_event", window.hide)
			window.set_position(WIN_POS_CENTER)
			window.set_title("Question")
			window.set_usize(200, 70)
			window.show()

			vbox = GtkVBox()
			window.add(vbox)
			vbox.show()

			filename_parts = string.split(self.file[current_page]["filename"], '/')
			if filename_parts[len(filename_parts) - 1] == '':
				filename = 'Untitled ' + str(current_page)
			else:
				filename = filename_parts[len(filename_parts) - 1]

			label = GtkLabel()
			label.set_text('"' + filename + '" has been modified.\nDo you wish to save it?')
			vbox.pack_start(label, expand=FALSE)
			label.show()

			hbox = GtkHBox()
			vbox.pack_start(hbox, expand=FALSE)
			hbox.show()

			yes_button = GtkButton("Yes")
			yes_button.connect("clicked", self.yes_save_file)
			hbox.pack_start(yes_button, expand=FALSE)
			yes_button.show()

			no_button = GtkButton("No")
			no_button.connect("clicked",self. no_save_file)
			hbox.pack_start(no_button, expand=FALSE)
			no_button.show()

			cancel_button = GtkButton("Cancel")
			cancel_button.connect("clicked", self.cancel_close_file)
			hbox.pack_start(cancel_button, expand=FALSE)
			cancel_button.show()
		else:
			self.close_file()

	def yes_save_file(self, *args):
		'''The user has selected to save the current tab.'''

		self.ask_close_window.hide()
		self.save_file()

	def no_save_file(self, *args):
		'''Close the current tab without saving.'''

		self.ask_close_window.hide()
		self.close_file()

	def cancel_close_file(self, *args):
		'''Do nothing.'''

		self.ask_close_window.hide()

	def close_file(self, *args):
		'''Remove the current tab, and create a new blank document if there are no other files open.'''

		current_page = self.notebook.get_current_page()
		self.notebook.remove_page(current_page)
		del self.file[current_page]

		#if len(self.file) == 0:
		#	self.new_file()

	def text_changed(self, *args):
		'''Called when the current tab text has changed.'''

		current_page = self.notebook.get_current_page()
		if self.file[current_page]["edited"] == 0:
			if self.file[current_page]["filename"] == '':
				filename_parts = string.split(self.file[current_page]["filename"], '/')
				self.file[current_page]["label_widget"].set_text('Untitled ' + str(current_page) + '*')
			else:
				filename_parts = string.split(self.file[current_page]["filename"], '/')
				self.file[current_page]["label_widget"].set_text(filename_parts[len(filename_parts) - 1] + '*')
			self.file[current_page]["edited"] = 1

	def cut_selection(self, *args):
		'''Cut the current selection.'''

		current_page = self.notebook.get_current_page()
		self.file[current_page]["text_widget"].cut_clipboard()

	def copy_selection(self, *args):
		'''Copy the current selection.'''

		current_page = self.notebook.get_current_page()
		self.file[current_page]["text_widget"].copy_clipboard()

	def paste_clipboard(self, *args):
		'''Paste the current clipboard.'''

		current_page = self.notebook.get_current_page()
		self.file[current_page]["text_widget"].paste_clipboard()

	def quit(self, *args):
		self.window.hide()
		mainquit()

	def mainloop(self, *args):
		mainloop()

sedit = Sedit()
sedit.mainloop()